# generated by datamodel-codegen:
#   filename:  feature.yaml

from __future__ import annotations

from pydantic import BaseModel, Field


class NPFeature(BaseModel):
    model: str = Field(..., example='gpt2-small')
    source: str = Field(..., example='0-res-jb')
    index: int = Field(..., example=14057)
