# generated by datamodel-codegen:
#   filename:  activation.yaml

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Field


class NPActivation(BaseModel):
    tokens: List[str] = Field(..., description='List of tokens for this text', example=['The', 'cat', 'sat'])
    values: List[float] = Field(
        ..., description='Activation values corresponding to each token', example=[0.5, 0.8, 0.2]
    )
